unit WEBURL;

interface
{WEB components, they are not displayed at component page!
 Register them in usual way. Components, listed here, are
 used in WEB paging formation with TMidasPageProducer.

 TURLShow - enables put hyperreference into MidasPage
   URLAddress - put URL like www.torry.ru
   URLText - put URL text like Torry Delphi Pages


 TImageShow - enables include image into MidasPage
   Picture - load bitmap (other graphic is not supported!)
   URLPath - pathinfo to directory, where final CGI (or ISAPI)
     application is inserted. For example, in one calls the
     created ISAPI DLL like: http://10.10.10.103/scripts/MyISAPI.dll,
     the property must contains value SCRIPTS


 By Sergei V. Trepalin, trep@ism.ac.ru
 Free. No warranty.}


uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, WebComp, MidItems, HTTPApp;

type
  TURLShow=class(TWEBDataDisplay)
  private
    FURLAddress:string;
    FURLText: string;
  protected
    function ControlContent(Options:TWebContentOptions):string; override;
  published
    property TabIndex;
    property URLAddress:string read FURLAddress write FURLAddress;
    property URLText:string read FURLText write FURLText;
  end;

  TImageShow=class(TWebDataDisplay)
  private
    FPicture:TPicture;
    FAction:TWebActionItem;
    FPath:string;
    procedure SetPicture(const Value: TPicture);
    procedure OnAction(Sender: TObject; Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
  protected
    function ControlContent(Options:TWebContentOptions):string; override;
  public
    constructor Create(AOwner:TComponent); override;
    destructor Destroy; override;
  published
    property Picture:TPicture read FPicture write SetPicture;
    property URLPath:string read FPath write FPath;
  end;

procedure Register;

implementation
uses JPeg;

procedure Register;
begin
  RegisterWEBComponents([TURLShow,TImageShow]);
end;

{ TURLShow }

function TURLShow.ControlContent(Options: TWebContentOptions): string;
var
  Attrs:string;
  Events:string;
  S:string;
begin
  Attrs:='';
  AddQuotedAttrib(Attrs,'NAME',GetHTMLControlName);
  AddIntAttrib(Attrs,'TABINDEX',TabIndex);
  AddQuotedAttrib(Attrs,'URLAddress',URLAddress);
  AddQuotedAttrib(Attrs,'URLText',URLText);
  if (not (coNoScript in Options.Flags)) then Events:=EventContent(Options);
  S:=FURLText;
  if length(S)=0 then S:=FURLAddress;
  if length(FURLAddress)>0 then Result:=Format('<a href="%s">%s</a>',[URLAddress,URLText]);
end;



{ TImageShow }

constructor TImageShow.Create(AOwner: TComponent);
var
  C:TComponent;
begin
  inherited;
  FPicture:=TPicture.Create;
  if not (csDesigning in ComponentState) then begin
    C:=Self;
    repeat C:=C.Owner until (C=nil) or (C is TWebModule);
    if C is TWebModule then begin
      FAction:=TWEBModule(C).Actions.Add;
      FAction.PathInfo:='/Pict'+IntToStr(integer(Self));
      FAction.OnAction:=OnAction;
    end;
  end;
end;

destructor TImageShow.Destroy;
begin
  FPicture.Free;
  inherited;
end;

function TImageShow.ControlContent(Options: TWebContentOptions): string;
var
  Attrs:string;
  Events:string;
  S:string;
  N:integer;
begin
  SetLength(S,MAX_PATH);
  N:=GetModuleFileName(hInstance,@S[1],MAX_PATH);
  SetLength(S,N);
  S:=ExtractFileName(S);
  Attrs:='';
  AddQuotedAttrib(Attrs,'NAME',GetHTMLControlName);
  if (not (coNoScript in Options.Flags)) then Events:=EventContent(Options);
  Result := Format('<IMG SRC="/'+FPath+'/%s/Pict'+IntToStr(integer(Self))+'" alt="[Picture]" border="0">',
  [S]);
end;

procedure TImageShow.SetPicture(const Value: TPicture);
begin
  FPicture.Assign(Value);
end;

procedure TImageShow.OnAction(Sender: TObject; Request: TWebRequest;
  Response: TWebResponse; var Handled: Boolean);
var
  Jpg: TJpegImage;
  S: TMemoryStream;
  B: TBitmap;
begin
  Jpg := TJpegImage.Create;
  try
    B := TBitmap.Create;
    try
      B.Assign(FPicture.Graphic);
      Jpg.Assign(B);
    finally
      B.Free;
    end;
    S := TMemoryStream.Create;
    Jpg.SaveToStream(S);
    S.Position := 0;
    Response.ContentType := 'image/jpeg';
    Response.ContentStream := S; // do not free the stream because the response
                                 // object will handle that task.
  finally
    Jpg.Free;
  end;
end;

end.
